#include "../http/gskhttprequest.h"
#include "../http/gskhttpresponse.h"
#include "../gskinit.h"
#include <string.h>

GskHttpHeader *
header_copy_through_buffers (GskHttpHeader *header)
{
  gboolean is_request = GSK_IS_HTTP_REQUEST (header);
  GskBuffer buffer1, buffer2;
  gpointer data1, data2;
  gsize len1, len2;
  GskHttpHeader *header1, *header2;
  gsk_buffer_construct (&buffer1);
  gsk_buffer_construct (&buffer2);

  gsk_http_header_to_buffer (header, &buffer1);
  len1 = buffer1.size;
  data1 = g_malloc (len1);
  gsk_buffer_peek (&buffer1, data1, len1);

  header1 = gsk_http_header_from_buffer (&buffer1, is_request, GSK_HTTP_PARSE_STRICT);
  g_assert (header1);
  g_assert (buffer1.size == 0);

  gsk_http_header_to_buffer (header1, &buffer2);
  len2 = buffer2.size;
  data2 = g_malloc (len2);
  gsk_buffer_peek (&buffer2, data2, len2);

  header2 = gsk_http_header_from_buffer (&buffer2, is_request, GSK_HTTP_PARSE_STRICT);
  g_assert (header2);
  g_assert (buffer2.size == 0);

  g_assert (len1 == len2);
  g_assert (memcmp (data1, data2, len1) == 0);
  g_object_unref (header1);
  g_free (data1);
  g_free (data2);
  return header2;
}

static GskHttpHeader *
header_from_string(gboolean is_request, const char *str)
{
  GskBuffer buffer;
  GskHttpHeader *header;
  gsk_buffer_construct (&buffer);
  gsk_buffer_append_foreign (&buffer, str, strlen(str), NULL, NULL);
  header = gsk_http_header_from_buffer (&buffer, is_request, 0);
  g_assert (header != NULL);
  gsk_buffer_destruct (&buffer);
  return header;
}

static gboolean
headers_equal (GskHttpHeader *h1, GskHttpHeader *h2)
{
  if ( ( (GSK_IS_HTTP_REQUEST (h1) ? 1 : 0)
       ^ (GSK_IS_HTTP_REQUEST (h2) ? 1 : 0) ) != 0)
    return FALSE;

  return TRUE;
}

int main(int argc, char **argv)
{
  GskHttpHeader *header0, *header1;
  GValue value;

  gsk_init (&argc, &argv, NULL);

  header0 = header_from_string (TRUE,
				"GET / HTTP/1.0\r\n"
				"User-Agent: foo\r\n"
				"\n");
  header1 = header_copy_through_buffers (header0);
  memset (&value, 0, sizeof(value));
  g_value_init (&value, G_TYPE_UINT);
  g_object_get_property (G_OBJECT (header1), "major-version", &value);
  g_assert (g_value_get_uint (&value) == 1);
  g_object_get_property (G_OBJECT (header1), "minor-version", &value);
  g_assert (g_value_get_uint (&value) == 0);
  g_value_unset (&value);
  g_value_init (&value, G_TYPE_STRING);
  g_object_get_property (G_OBJECT (header1), "user-agent", &value);
  g_assert (strcmp (g_value_get_string (&value), "foo") == 0);
  g_object_get_property (G_OBJECT (header1), "referrer", &value);
  g_assert (g_value_get_string (&value) == NULL);
  g_value_unset (&value);
  g_value_init (&value, GSK_TYPE_HTTP_CONNECTION);
  g_object_get_property (G_OBJECT (header1), "connection", &value);
  g_assert (g_value_get_enum (&value) == GSK_HTTP_CONNECTION_NONE);
  g_value_unset (&value);
  g_assert (headers_equal (header0, header1));
  g_assert (header1->http_major_version == 1
         && header1->http_minor_version == 0);
  g_assert (gsk_http_header_get_connection (header1) == GSK_HTTP_CONNECTION_CLOSE);
  g_assert (gsk_http_header_get_connection_type (header1) == GSK_HTTP_CONNECTION_NONE);
  g_object_unref (header0);
  g_object_unref (header1);

  header0 = header_from_string (TRUE,
				"GET / HTTP/1.1\r\n"
				"Referer: http://whatever.com\r\n"
				"User-Agent: foo\r\n"
				"Transfer-Encoding: chunked\r\n"
				"\n");
  header1 = header_copy_through_buffers (header0);
  g_assert (headers_equal (header0, header1));
  g_assert (header1->http_major_version == 1
         && header1->http_minor_version == 1);

  g_value_init (&value, G_TYPE_UINT);
  g_object_get_property (G_OBJECT (header1), "major-version", &value);
  g_assert (g_value_get_uint (&value) == 1);
  g_object_get_property (G_OBJECT (header1), "minor-version", &value);
  g_assert (g_value_get_uint (&value) == 1);
  g_value_unset (&value);
  g_value_init (&value, G_TYPE_STRING);
  g_object_get_property (G_OBJECT (header1), "user-agent", &value);
  g_assert (strcmp (g_value_get_string (&value), "foo") == 0);
  g_object_get_property (G_OBJECT (header1), "referrer", &value);
  g_assert (strcmp (g_value_get_string (&value), "http://whatever.com") == 0);
  g_value_unset (&value);
  g_value_init (&value, GSK_TYPE_HTTP_CONNECTION);
  g_object_get_property (G_OBJECT (header1), "connection", &value);
  g_assert (g_value_get_enum (&value) == GSK_HTTP_CONNECTION_NONE);
  g_value_unset (&value);
  g_value_init (&value, GSK_TYPE_HTTP_TRANSFER_ENCODING);
  g_object_get_property (G_OBJECT (header1), "transfer-encoding", &value);
  g_assert (g_value_get_enum (&value) == GSK_HTTP_TRANSFER_ENCODING_CHUNKED);
  g_value_unset (&value);
  g_assert (gsk_http_header_get_connection (header1) == GSK_HTTP_CONNECTION_KEEPALIVE);
  g_assert (gsk_http_header_get_connection_type (header1) == GSK_HTTP_CONNECTION_NONE);
  g_object_unref (header0);
  g_object_unref (header1);

  {
    GskHttpRequest *request;
    header0 = header_from_string (TRUE,
				  "GET / HTTP/1.1\r\n"
				  "Accept-Language: swahili;q=4, cherokee;q=2, estonian\r\n"
				  "\n");
    header1 = header_copy_through_buffers (header0);
    g_assert (GSK_IS_HTTP_REQUEST (header0));
    g_assert (headers_equal (header0, header1));
    request = GSK_HTTP_REQUEST (header1);
    g_assert (request->accept_languages != NULL);
    g_assert (request->accept_languages->next != NULL);
    g_assert (request->accept_languages->next->next != NULL);
    g_assert (request->accept_languages->next->next->next == NULL);
    g_assert (strcmp (request->accept_languages->language, "swahili") == 0);
    g_assert (request->accept_languages->quality == 4);
    g_assert (strcmp (request->accept_languages->next->language, "cherokee") == 0);
    g_assert (request->accept_languages->next->quality == 2);
    g_assert (strcmp (request->accept_languages->next->next->language, "estonian") == 0);
    g_assert (request->accept_languages->next->next->quality == -1);
    g_object_unref (header0);
    g_object_unref (header1);
  }
  {
    GskHttpRequest *request;
    header0 = header_from_string (TRUE,
				  "GET / HTTP/1.1\r\n"
				  "Accept-Language: en-us\r\n"
				  "\n");
    header1 = header_copy_through_buffers (header0);
    g_assert (GSK_IS_HTTP_REQUEST (header0));
    g_assert (headers_equal (header0, header1));
    request = GSK_HTTP_REQUEST (header1);
    g_assert (request->accept_languages->quality == -1.0);
    g_assert (request->accept_languages != NULL);
    g_assert (request->accept_languages->next == NULL);
    g_assert (strcmp (request->accept_languages->language, "en-us") == 0);
    g_object_unref (header0);
    g_object_unref (header1);
  }
  {
    GskHttpRequest *request;
    header0 = header_from_string (TRUE,
				  "GET / HTTP/1.1\r\n"
				  "Accept-Language: en-us,en;q=0.5\r\n"
				  "\n");
    header1 = header_copy_through_buffers (header0);
    g_assert (GSK_IS_HTTP_REQUEST (header0));
    g_assert (headers_equal (header0, header1));
    request = GSK_HTTP_REQUEST (header1);
    g_assert (request->accept_languages != NULL);
    g_assert (request->accept_languages->quality == -1.0);
    g_assert (request->accept_languages->next != NULL);
    g_assert (request->accept_languages->next->quality == 0.5);
    g_assert (request->accept_languages->next->next == NULL);
    g_assert (strcmp (request->accept_languages->language, "en-us") == 0);
    g_object_unref (header0);
    g_object_unref (header1);
  }

  return 0;
}
